// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// HelloWorld_db.h was generated from the DDL file HelloWorld_db.ddl.
#include "HelloWorld_db.h"

// HelloWorld_db_api.h was generated from the DDL file HelloWorld_db.ddl. with 
// the -cpp option
#include "HelloWorld_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - Hello World for C++
// Each time you run this example, a new record is created in the
// database, then all existing records are read from the database
// and printed.

// The DDL (Database Definition Language) is in HelloWorld_db.ddl. The database
// has been pre-built and initialized.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_records
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'HelloWorld_db' in this
// project directory.

int32_t main(void)
{
    Db_HelloWorld_db   db;
    struct HelloRecord HelloRecord_input;
    struct HelloRecord HelloRecord_output;
    Cursor_HelloRecord HelloRecord_cursor;

    // Open the database
    try
    {
        db = db.Open("HelloWorld_db");
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening HelloWorld_db database" << endl;
        return EXIT_FAILURE;
    }

    // Create a new record in the database
    try
    {
        strncpy(HelloRecord_input.message, "Hello World", SIZEOF_MESSAGE);
        HelloRecord_cursor = db.New_HelloRecord_recordWithFieldValues(&HelloRecord_input);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding record" << endl;
        return EXIT_FAILURE;
    }

    // scan and read all created records
    // one additional record will be created each time you run this example
    HelloRecord_cursor = db.Get_HelloRecord_records();
    while (!HelloRecord_cursor.AfterLast())
    {
        HelloRecord_cursor.GetFieldValues(&HelloRecord_output);
        cout << HelloRecord_output.message << endl;
        HelloRecord_cursor++;
    }
    
    cout << "Press Enter to terminate" << endl;
    getchar();

    return EXIT_SUCCESS;
}
