#ifndef HELLOWORLD_DB_API_H
#define HELLOWORLD_DB_API_H

/* RDM 11.0.0.1000 */

#include "Db.h"
#include "Cursor.h"
#include "HelloWorld_db.h"

namespace RDM_CPP {

class Cursor_HelloRecord;

/*! \brief The HelloWorld_db Db class
 *
 * This class provides the implementation for the HelloWorld_db database
*/
class Db_HelloWorld_db : public Db {
public:
    /*! \brief Open the HelloWorld_db database using the default name
     *
     * \param mode The open mode to use
     * \param dbd_array The dbd_array generated by the ddlp -c option
     * \param dbdsize sizeof(dbd_array)
     * \return A validated Db_HelloWorld_db object
     */
    static Db_HelloWorld_db Open (char *mode="s", const void *dbd_array=NULL, int32_t dbdsize=0) throw (rdm_exception&);

    /*! \brief Open the HelloWorld_db database using the default name
     *
     * \param mode The open mode to use
     * \param dbd_array The dbd_array generated by the ddlp -c option
     * \param dbdsize sizeof(dbd_array)
     * \return A validated Db_HelloWorld_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_HelloWorld_db Open (wchar_t *mode=L"s", const void *dbd_array=NULL, int32_t dbdsize=0) throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the HelloWorld_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \param dbd_array The dbd_array generated by the ddlp -c option
     * \param dbdsize sizeof(dbd_array)
     * \return A validated Db_HelloWorld_db object
     */
    static Db_HelloWorld_db Open (const char *name, const char *mode="s", const void *dbd_array=NULL, int32_t dbdsize=0) throw (rdm_exception&);

    /*! \brief Open the HelloWorld_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \param dbd_array The dbd_array generated by the ddlp -c option
     * \param dbdsize sizeof(dbd_array)
     * \return A validated Db_HelloWorld_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_HelloWorld_db Open (const wchar_t *name, const wchar_t *mode=L"s", const void *dbd_array=NULL, int32_t dbdsize=0) throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_HelloWorld_db Default Constructor
    Db_HelloWorld_db (void) throw ();
    /*! \brief Db_HelloWorld_db Constructor
     *
     * \param db A Db object used to create the Db_HelloWorld_db
     */
    Db_HelloWorld_db (const Db& db) throw ();
    //! Db_HelloWorld_db Copy Constructor
    Db_HelloWorld_db (const Db_HelloWorld_db&) throw ();

    //! Db_HelloWorld_db assignment operator
    Db_HelloWorld_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_HelloWorld_db assignment operator
    Db_HelloWorld_db& operator = (const Db_HelloWorld_db&) throw ();

    /*! \brief Db_HelloWorld_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_HelloWorld_db ();

    /*! \brief Create a new HelloRecord record
     *
     * \returns A singleton Cursor_HelloRecord object with default field values
     *
     * Create a new HelloRecord record using default values
     */
    Cursor_HelloRecord New_HelloRecord_record (void) const throw (rdm_exception&);

    /*! \brief Create a new HelloRecord record
     *
     * \param fields The field values for the new HelloRecord record
     * \returns A singleton Cursor_HelloRecord object with the specified field values
     *
     * Create a new HelloRecord record using the specified values
     */
    Cursor_HelloRecord New_HelloRecord_recordWithFieldValues (struct HelloRecord *fields) const throw (rdm_exception&);

    /*! \brief Obtain HelloRecord cursor position at the first HelloRecord record
     * \returns HelloRecord cursor positioned at the first HelloRecord record
     *
     * Create a new HelloRecord cursor for doing a table scan of the HelloRecord records.
     * The cursor is positioned at the first HelloRecord record
     */
    Cursor_HelloRecord Get_HelloRecord_records (void) const throw (rdm_exception&);

private:
    /*! \brief Db_HelloWorld_db reference operator */
    Db_HelloWorld_db* operator & (void);
    /*! \brief Db_HelloWorld_db constructor */
    Db_HelloWorld_db (IDb *idb);
    /*! \brief Get the Db_HelloWorld_db dbType
     * \return The Db_HelloWorld_db dbType
     */
    static dbType Type (void);
};

/*! \brief The HelloRecord Cursor class
 *
 * This class provides the implementation for the HelloRecord Cursor
*/
class Cursor_HelloRecord : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_HelloRecord objects
     */
    static recordType Type (void);

    /*! Cursor_HelloRecord Default Constructor */
    Cursor_HelloRecord (void) throw ();
    /*! \brief Cursor_HelloRecord Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_HelloRecord object
     */
    Cursor_HelloRecord (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_HelloRecord Copy Constructor */
    Cursor_HelloRecord (const Cursor_HelloRecord&) throw ();

    /*! \brief Cursor_HelloRecord assignment operator */
    Cursor_HelloRecord& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_HelloRecord assignment operator */
    Cursor_HelloRecord& operator = (const Cursor_HelloRecord& cursor) throw ();

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct HelloRecord *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct HelloRecord *fields) const throw (rdm_exception&);

    /*! \brief Set the message field value for the current record
     * \param message The field value that will be written to the record at the current cursor position
     */
    void Set_message (const char message_val[32]) const throw (rdm_exception&);

    /*! \brief Get the message field value for the current record
     * \param message A pointer where the field value from the current cursor position will be written
     */
    void Get_message (char message_val[32]) const throw (rdm_exception&);

    /*! \brief Cursor_HelloRecord Destructor
     *
     * Release the cursor
     */
    ~Cursor_HelloRecord ();

private:
    friend class Db_HelloWorld_db;
};

}
#endif    /* HELLOWORLD_DB_API_H */
